@@ -47,6 +47,9 @@ class NucleotideAnalysis(metaclass=ABCMeta):
     def init_tools(self):
         super().init_tools()
 
+    def reset(self):
+        super().reset()
+
 
 class ProteinAnalysis:
 
@@ -72,6 +75,7 @@ class ProteinAnalysis:
         "W",
         "R",
         "G",
+        "*",
     ]
     NUCL_LETTERS = ["A", "C", "T", "G", "N"]
 
@@ -114,7 +118,13 @@ class BLASTAnalysis(metaclass=ABCMeta):
                 )
             )
 
+    def reset(self):
+        super().reset()
+        self.mkblast_runner.reset()
+        self.tblastn_runner.reset()
+
     def _run_mkblast(self):
+        self.mkblast_runner.configure_runner()
         if self.restart and self.mkblast_runner.check_previous_completed_run():
             logger.info(
                 "Skipping makeblastdb as BLAST DB already exists at {}".format(
